const mongoose = require('mongoose');
mongoose.connect(process.env.MONGO_URL, { useMongoClient: true });

const express = require('express');
const Badge = require('openbadge');
const parse = require('git-url-parse');
const path = require('path');
const serveStatic = require('serve-static');

const app = express();

const Coverage = require('./lib/coverage');
const { parseBody } = require('./lib/util');
const port = process.env.PORT || 8080;

app.post('/api/v1/upload', parseBody, (req, res) => {
  let {
    git,
    run_at,
    source_files,
    service_job_id,
    service_pull_request,
    service_name
  } = req.body;

  // Make sure the remote url is set correctly
  git.remotes.url = parse(parse(git.remotes.url).toString("ssh")).toString("https");

  Coverage.save({
      source_files,
      git,
      run_at,
      service_job_id,
      service_pull_request,
      service_name
  }).then((result) => {
    res.send({ result });
  }).catch((err) => {
    res.status(500);
    res.send({ error: err });
  });
});

app.get('/api/v1/coverage', (req, res) => {
  Coverage.get()
    .then((coverages) => {
      res.send(coverages);
    }).catch(function(err) {
      res.status(500);
      res.send({error: err});
    });
});

app.get('/api/v1/coverage/:service/:owner', (req, res) => {
  const { service, owner } = req.params;

  Coverage.get(new RegExp(`${service.replace(/%2E/g, '.')}.*/${owner}/`))
    .then((coverages) => {
      res.send(coverages);
    }).catch(function(err) {
      res.status(500);
      res.send({error: err});
    });
});

app.get('/api/v1/coverage/:service/:owner/:repo', (req, res) => {
  const { service, owner, repo } = req.params;

  Coverage.get(new RegExp(`${service.replace(/%2E/g, '.')}.*/${owner}/${repo}`))
    .then((coverages) => {
      res.send(coverages);
    }).catch(function(err) {
      res.status(500);
      res.send({error: err});
    });
});

app.get('/badge/:service/:owner/:repo.svg', (req, res) => {
  const { service, owner, repo } = req.params;

  Coverage.get(new RegExp(`${service.replace(/%2E/g, '.')}.*/${owner}/${repo}`))
    .then((coverages) => {
      const coverage = coverages[0];
      const { history } = coverage;
      const lastRun = history[history.length - 1];
      const { lines } = lastRun.source_files[0];
      const percentage = parseInt((lines.hit / lines.found) * 100);
      Badge({text: ['coverage', `${percentage}%`] }, function(err, badge) {
          if(err) { throw new Error(err); }
          res.set('Content-Type', 'image/svg+xml; charset=utf-8');
          res.send(badge);
      });
    }).catch(function() {
        Badge({ color: { right: "#b63b3b" }, text: ['coverage', 'not found'] }, function(err, badge) {
            if(err) { throw new Error(err); }
            res.set('Content-Type', 'image/svg+xml; charset=utf-8');
            res.send(badge);
        });
    });
});

app.use(serveStatic(path.resolve(__dirname, 'dist')));

app.get('*', (req, res) => {
  res.sendFile(path.resolve(__dirname, 'dist', 'index.html'));
});

app.listen(port, function () {
  console.log(`node-coverage-server is listening on http://localhost:${port}`); // eslint-disable-line
});
